// 
// example of Monte Carlo simulation method
//
// evaluate \pi 
//

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <ctime>
using namespace std;

const long NEXP = 100000000;	// max number of experiments

// generate random number <0..1)
inline double Random()
{
    return (double) rand() / RAND_MAX;
}

int main()
{
    long N = 100;		// for print only
    long S = 0;
    srand(time(0));
    clock_t start = clock();
    for (int i = 0; i <= NEXP; i++) {
	double x = Random();
	double y = Random();
	if ((x * x + y * y) < 1.0)
	    S++;
	if (i == N) {		// print results
	    double pi = 4.0 * (double) S / i;
	    cout << "pi = " << setprecision(8) << pi << "\t";
	    cout << "err = " << setprecision(2) << (pi - M_PI)/M_PI*100 << "%\t";
	    cout << "i = " << i << endl;
	    N *= 10;
	}
    }
    cout << "Time: ";
    cout << setprecision(4) << double(clock()-start)/CLOCKS_PER_SEC;
    cout << " s" << endl;
}

