// 
// example of Monte Carlo simulation method
// integral of volume
//
// Analytical solution: 4/3 pi r^3
//

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <ctime>
using namespace std;

const long NEXP = 10000000;	// max number of experiments

// generate random number 0..1
inline double Random()
{
    return (double) rand() / RAND_MAX;
}

int main()
{
    srand(time(0));
    long N = 100;		// for print only
    long S = 0;
    clock_t start = clock();

    cout << endl;
    for (int i = 0; i <= NEXP; i++) {
	double x = Random();
	double y = Random();
	double z = Random();
	if (x*x+y*y+z*z<1)
	    S++;
	if (i == N) {		// print results
	    double probability = (double) S / i;
	    double result = 8 * probability;
	    cout << "result = " << result << "\t";
	    cout << "(" << i << " experiments)" <<  endl;
	    N *= 10;
	}
    }
    cout << endl;
    cout << "Exact solution: " << 4.0/3.0*M_PI << endl;
    cout << "Time: ";
    cout << setprecision(4) << double(clock()-start)/CLOCKS_PER_SEC;
    cout << " s" << endl;
}

