// 
// example of Monte Carlo simulation method
// partial dif. equation.
//

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <cassert>
using namespace std;

const long NEXP = 1000000;	// max number of experiments

// generate random number 0..1
inline double Random()
{
    return (double) rand() / RAND_MAX;
}

const int N = 5;
int bound[N][N] = {
    {1, 1, 1, 1, 1,},
    {1, 0, 0, 0, 1,},
    {1, 0, 0, 0, 1,},
    {1, 0, 0, 0, 1,},
    {1, 1, 1, 1, 1,},
};
double b[N][N] = {		// bounds AND analytic solution
    { 3,  4, -7, 10,  10,},
    { 2,  3,  2,  7,  10,},
    {-2,  4,  5,  6,  11,},
    { 8, 10,  8,  1, -10,},
    {14, 20, 16,  0,  -5,},
};

double v[N][N];			// results

// array for steps in four directions
const struct { int x; int y; } move[4] = {
    {  1,  0 }, 
    {  0,  1 }, 
    { -1,  0 }, 
    {  0, -1 },
};

double solve(int x0, int y0)
{
    double result;
    long n = 100;		// for print only
    double S = 0;
    long i;
    for (i = 0; i < NEXP; i++) {
	int x = x0;
	int y = y0;
	while (!bound[y][x]) {
	    int r = int (Random() * 4);
	    x += move[r].x;
	    y += move[r].y;
	}
	S += b[y][x];

	if (i == n) {		// print results
	    result = S / i;
#if 0
	    // for testing:
	    cout << "result = " << result << "\t";
	    cout << "(" << i << " experiments)" << endl;
#endif
	    n *= 10;
	}
    }
    return result;
}

int main()
{
    srand(time(0));
    clock_t start = clock();
    cout << endl;
    for (int y0 = 0; y0 < N; y0++)
	for (int x0 = 0; x0 < N; x0++)
	    if (bound[y0][x0])	// initialize bounds
		v[y0][x0] = b[y0][x0];
    for (int y0 = 1; y0 < N - 1; y0++)
	for (int x0 = 1; x0 < N - 1; x0++)
	    v[y0][x0] = solve(x0, y0);	// compute solutions
    cout << endl;
    cout << "#--------------------------------------------------" << endl;
    for (int y0 = 0; y0 < N; y0++) {
	for (int x0 = 0; x0 < N; x0++)
	    cout << v[y0][x0] << '\n';	// \t for human readable output
	cout << endl;
    }
    cout << "#--------------------------------------------------" << endl;
    cout << endl;
    cout << "#Time: ";
    cout << setprecision(4) << double (clock() - start) / CLOCKS_PER_SEC;
    cout << " s" << endl;
}
